@@ -32,8 +32,6 @@
 #include <crt_externs.h>
 #endif
 
-#undef G_DISABLE_DEPRECATED
-
 #include "gcontenttypeprivate.h"
 #include "gdesktopappinfo.h"
 #include "gfile.h"
@@ -673,7 +671,7 @@ g_desktop_app_info_get_categories (GDesktopAppInfo *info)
  *
  * Gets the keywords from the desktop file.
  *
- * Returns: The value of the X-GNOME-Keywords key
+ * Returns: (transfer none): The value of the X-GNOME-Keywords key
  *
  * Since: 2.32
  */
