@@ -2,12 +2,10 @@
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  */
@@ -32,12 +32,15 @@
 
 
 /**
- * GFilenameCompleter:
+ * SECTION:gfilenamecompleter
+ * @short_description: Filename Completer
+ * @include: gio/gio.h
  * 
  * Completes partial file and directory names given a partial string by
  * looking in the file system for clues. Can return a list of possible 
  * completion strings for widget implementations.
- */
+ * 
+ **/
 
 enum {
   GOT_COMPLETION_DATA,
@@ -67,7 +70,7 @@ struct _GFilenameCompleter {
   LoadBasenamesData *basename_loader;
 };
 
-G_DEFINE_TYPE (GFilenameCompleter, g_filename_completer, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GFilenameCompleter, g_filename_completer, G_TYPE_OBJECT);
 
 static void cancel_load_basenames (GFilenameCompleter *completer);
 
@@ -83,7 +86,8 @@ g_filename_completer_finalize (GObject *object)
   if (completer->basenames_dir)
     g_object_unref (completer->basenames_dir);
 
-  g_list_free_full (completer->basenames, g_free);
+  g_list_foreach (completer->basenames, (GFunc)g_free, NULL);
+  g_list_free (completer->basenames);
 
   G_OBJECT_CLASS (g_filename_completer_parent_class)->finalize (object);
 }
@@ -104,7 +108,7 @@ g_filename_completer_class_init (GFilenameCompleterClass *klass)
 					  G_SIGNAL_RUN_LAST,
 					  G_STRUCT_OFFSET (GFilenameCompleterClass, got_completion_data),
 					  NULL, NULL,
-					  NULL,
+					  g_cclosure_marshal_VOID__VOID,
 					  G_TYPE_NONE, 0);
 }
 
@@ -151,7 +155,8 @@ load_basenames_data_free (LoadBasenamesData *data)
   g_object_unref (data->cancellable);
   g_object_unref (data->dir);
   
-  g_list_free_full (data->basenames, g_free);
+  g_list_foreach (data->basenames, (GFunc)g_free, NULL);
+  g_list_free (data->basenames);
   
   g_free (data);
 }
@@ -238,7 +243,8 @@ got_more_files (GObject *source_object,
       
       if (data->completer->basenames_dir)
 	g_object_unref (data->completer->basenames_dir);
-      g_list_free_full (data->completer->basenames, g_free);
+      g_list_foreach (data->completer->basenames, (GFunc)g_free, NULL);
+      g_list_free (data->completer->basenames);
       
       data->completer->basenames_dir = g_object_ref (data->dir);
       data->completer->basenames = data->basenames;
@@ -275,9 +281,10 @@ got_enum (GObject *source_object,
 
       if (data->completer->basenames_dir)
 	g_object_unref (data->completer->basenames_dir);
-      g_list_free_full (data->completer->basenames, g_free);
+      g_list_foreach (data->completer->basenames, (GFunc)g_free, NULL);
+      g_list_free (data->completer->basenames);
 
-      /* Mark up-to-date with no basenames */
+      /* Mark uptodate with no basenames */
       data->completer->basenames_dir = g_object_ref (data->dir);
       data->completer->basenames = NULL;
       data->completer->basenames_are_escaped = data->should_escape;
@@ -342,13 +349,11 @@ init_completion (GFilenameCompleter *completer,
 		 char **basename_out)
 {
   gboolean should_escape;
-  GFile *file = NULL, *parent = NULL;
+  GFile *file, *parent;
   char *basename;
   char *t;
-  size_t len;
-  GList *basenames;
+  int len;
 
-  basenames = NULL;
   *basename_out = NULL;
   
   should_escape = ! (g_path_is_absolute (initial_text) || *initial_text == '~');
@@ -357,20 +362,23 @@ init_completion (GFilenameCompleter *completer,
   
   if (len > 0 &&
       initial_text[len - 1] == '/')
-    goto out;
+    return NULL;
   
   file = g_file_parse_name (initial_text);
   parent = g_file_get_parent (file);
   if (parent == NULL)
-    goto out;
+    {
+      g_object_unref (file);
+      return NULL;
+    }
 
   if (completer->basenames_dir == NULL ||
       completer->basenames_are_escaped != should_escape ||
       !g_file_equal (parent, completer->basenames_dir))
     {
       schedule_load_basenames (completer, parent, should_escape);
-
-      goto out;
+      g_object_unref (file);
+      return NULL;
     }
   
   basename = g_file_get_basename (file);
@@ -387,17 +395,12 @@ init_completion (GFilenameCompleter *completer,
       g_free (t);
       
       if (basename == NULL)
-        goto out;
+	return NULL;
     }
 
-  basenames = completer->basenames;
   *basename_out = basename;
 
-out:
-  g_clear_object (&file);
-  g_clear_object (&parent);
-
-  return basenames;
+  return completer->basenames;
 }
 
 /**
@@ -407,9 +410,9 @@ out:
  *
  * Obtains a completion for @initial_text from @completer.
  *  
- * Returns: (nullable) (transfer full): a completed string, or %NULL if no
- *     completion exists. This string is not owned by GIO, so remember to g_free()
- *     it when finished.
+ * Returns: a completed string, or %NULL if no completion exists. 
+ *     This string is not owned by GIO, so remember to g_free() it 
+ *     when finished.
  **/
 char *
 g_filename_completer_get_completion_suffix (GFilenameCompleter *completer,
