@@ -75,6 +75,7 @@
 #include <gio/gfilteroutputstream.h>
 #include <gio/gicon.h>
 #include <gio/ginetaddress.h>
+#include <gio/ginetaddressmask.h>
 #include <gio/ginetsocketaddress.h>
 #include <gio/ginitable.h>
 #include <gio/ginputstream.h>
@@ -91,6 +92,7 @@
 #include <gio/gmountoperation.h>
 #include <gio/gnativevolumemonitor.h>
 #include <gio/gnetworkaddress.h>
+#include <gio/gnetworkmonitor.h>
 #include <gio/gnetworkservice.h>
 #include <gio/goutputstream.h>
 #include <gio/gpermission.h>
@@ -102,6 +104,7 @@
 #include <gio/gproxyresolver.h>
 #include <gio/gresolver.h>
 #include <gio/gseekable.h>
+#include <gio/gsettingsschema.h>
 #include <gio/gsettings.h>
 #include <gio/gsimpleasyncresult.h>
 #include <gio/gsimplepermission.h>
