@@ -7,7 +7,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -31,7 +33,7 @@
 #include "gnetworkaddress.h"
 #include "gnetworkingprivate.h"
 #include "gresolver.h"
-#include "gtask.h"
+#include "gsimpleasyncresult.h"
 #include "gsocketaddressenumerator.h"
 #include "gsocketconnectable.h"
 #include "gsrvtarget.h"
@@ -52,7 +54,7 @@
  * address families.
  *
  * See #GSrvTarget for more information about SRV records, and see
- * #GSocketConnectable for an example of using the connectable
+ * #GSocketConnectable for and example of using the connectable
  * interface.
  */
 
@@ -89,10 +91,8 @@ static void g_network_service_get_property (GObject      *object,
 static void                      g_network_service_connectable_iface_init       (GSocketConnectableIface *iface);
 static GSocketAddressEnumerator *g_network_service_connectable_enumerate        (GSocketConnectable      *connectable);
 static GSocketAddressEnumerator *g_network_service_connectable_proxy_enumerate  (GSocketConnectable      *connectable);
-static gchar                    *g_network_service_connectable_to_string        (GSocketConnectable      *connectable);
 
 G_DEFINE_TYPE_WITH_CODE (GNetworkService, g_network_service, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (GNetworkService)
                          G_IMPLEMENT_INTERFACE (G_TYPE_SOCKET_CONNECTABLE,
                                                 g_network_service_connectable_iface_init))
 
@@ -117,6 +117,8 @@ g_network_service_class_init (GNetworkServiceClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GNetworkServicePrivate));
+
   gobject_class->set_property = g_network_service_set_property;
   gobject_class->get_property = g_network_service_get_property;
   gobject_class->finalize = g_network_service_finalize;
@@ -160,13 +162,13 @@ g_network_service_connectable_iface_init (GSocketConnectableIface *connectable_i
 {
   connectable_iface->enumerate = g_network_service_connectable_enumerate;
   connectable_iface->proxy_enumerate = g_network_service_connectable_proxy_enumerate;
-  connectable_iface->to_string = g_network_service_connectable_to_string;
 }
 
 static void
 g_network_service_init (GNetworkService *srv)
 {
-  srv->priv = g_network_service_get_instance_private (srv);
+  srv->priv = G_TYPE_INSTANCE_GET_PRIVATE (srv, G_TYPE_NETWORK_SERVICE,
+                                           GNetworkServicePrivate);
 }
 
 static void
@@ -243,7 +245,7 @@ g_network_service_get_property (GObject    *object,
  * @protocol, and @domain. This will initially be unresolved; use the
  * #GSocketConnectable interface to resolve it.
  *
- * Returns: (transfer full) (type GNetworkService): a new #GNetworkService
+ * Return value: (transfer full): a new #GNetworkService
  *
  * Since: 2.22
  */
@@ -265,7 +267,7 @@ g_network_service_new (const gchar *service,
  *
  * Gets @srv's service name (eg, "ldap").
  *
- * Returns: @srv's service name
+ * Return value: @srv's service name
  *
  * Since: 2.22
  */
@@ -283,7 +285,7 @@ g_network_service_get_service (GNetworkService *srv)
  *
  * Gets @srv's protocol name (eg, "tcp").
  *
- * Returns: @srv's protocol name
+ * Return value: @srv's protocol name
  *
  * Since: 2.22
  */
@@ -302,7 +304,7 @@ g_network_service_get_protocol (GNetworkService *srv)
  * Gets the domain that @srv serves. This might be either UTF-8 or
  * ASCII-encoded, depending on what @srv was created with.
  *
- * Returns: @srv's domain name
+ * Return value: @srv's domain name
  *
  * Since: 2.22
  */
@@ -318,10 +320,10 @@ g_network_service_get_domain (GNetworkService *srv)
  * g_network_service_get_scheme:
  * @srv: a #GNetworkService
  *
- * Gets the URI scheme used to resolve proxies. By default, the service name
+ * Get's the URI scheme used to resolve proxies. By default, the service name
  * is used as scheme.
  *
- * Returns: @srv's scheme name
+ * Return value: @srv's scheme name
  *
  * Since: 2.26
  */
@@ -352,7 +354,8 @@ g_network_service_set_scheme (GNetworkService *srv,
 {
   g_return_if_fail (G_IS_NETWORK_SERVICE (srv));
 
-  g_free (srv->priv->scheme);
+  if (srv->priv->scheme)
+    g_free (srv->priv->scheme);
   srv->priv->scheme = g_strdup (scheme);
 
   g_object_notify (G_OBJECT (srv), "scheme");
@@ -392,6 +395,9 @@ typedef struct {
 
   GError *error;
 
+  /* For async operation */
+  GCancellable *cancellable;
+  GSimpleAsyncResult *result;
 } GNetworkServiceAddressEnumerator;
 
 typedef struct {
@@ -445,7 +451,7 @@ g_network_service_address_enumerator_next (GSocketAddressEnumerator  *enumerator
     {
       if (srv_enum->addr_enum == NULL && srv_enum->t)
         {
-          GError *my_error = NULL;
+          GError *error = NULL;
           gchar *uri;
           gchar *hostname;
           GSocketConnectable *addr;
@@ -465,27 +471,21 @@ g_network_service_address_enumerator_next (GSocketAddressEnumerator  *enumerator
               continue;
             }
 
-          uri = g_uri_join (G_URI_FLAGS_NONE,
-                            g_network_service_get_scheme (srv_enum->srv),
-                            NULL,
-                            hostname,
-                            g_srv_target_get_port (target),
-                            "",
-                            NULL,
-                            NULL);
+          uri = _g_uri_from_authority (g_network_service_get_scheme (srv_enum->srv),
+                                       hostname,
+                                       g_srv_target_get_port (target),
+                                       NULL);
           g_free (hostname);
 
           addr = g_network_address_parse_uri (uri,
                                               g_srv_target_get_port (target),
-                                              &my_error);
+                                              &error);
           g_free (uri);
 
           if (addr == NULL)
             {
               if (srv_enum->error == NULL)
-                srv_enum->error = my_error;
-              else
-                g_error_free (my_error);
+                srv_enum->error = error;
               continue;
             }
 
@@ -498,18 +498,18 @@ g_network_service_address_enumerator_next (GSocketAddressEnumerator  *enumerator
 
       if (srv_enum->addr_enum)
         {
-          GError *my_error = NULL;
+          GError *error = NULL;
 
           ret = g_socket_address_enumerator_next (srv_enum->addr_enum,
                                                   cancellable,
-                                                  &my_error);
+                                                  &error);
 
-          if (my_error)
+          if (error)
             {
               if (srv_enum->error == NULL)
-                srv_enum->error = my_error;
+                srv_enum->error = error;
               else
-                g_error_free (my_error);
+                g_error_free (error);
             }
 
           if (!ret)
@@ -530,13 +530,13 @@ g_network_service_address_enumerator_next (GSocketAddressEnumerator  *enumerator
   return ret;
 }
 
-static void next_async_resolved_targets   (GObject      *source_object,
-                                           GAsyncResult *result,
-                                           gpointer      user_data);
-static void next_async_have_targets       (GTask        *srv_enum);
-static void next_async_have_address       (GObject      *source_object,
-                                           GAsyncResult *result,
-                                           gpointer      user_data);
+static void next_async_resolved_targets   (GObject                          *source_object,
+                                           GAsyncResult                     *result,
+                                           gpointer                          user_data);
+static void next_async_have_targets       (GNetworkServiceAddressEnumerator *srv_enum);
+static void next_async_have_address       (GObject                          *source_object,
+                                           GAsyncResult                     *result,
+                                           gpointer                          user_data);
 
 static void
 g_network_service_address_enumerator_next_async (GSocketAddressEnumerator  *enumerator,
@@ -546,10 +546,15 @@ g_network_service_address_enumerator_next_async (GSocketAddressEnumerator  *enum
 {
   GNetworkServiceAddressEnumerator *srv_enum =
     G_NETWORK_SERVICE_ADDRESS_ENUMERATOR (enumerator);
-  GTask *task;
 
-  task = g_task_new (enumerator, cancellable, callback, user_data);
-  g_task_set_source_tag (task, g_network_service_address_enumerator_next_async);
+  g_return_if_fail (srv_enum->result == NULL);
+
+  srv_enum->result = g_simple_async_result_new (G_OBJECT (enumerator),
+                                                callback, user_data,
+                                                g_network_service_address_enumerator_next_async);
+
+  if (cancellable)
+    srv_enum->cancellable = g_object_ref (cancellable);
 
   /* If we haven't yet resolved srv, do that */
   if (!srv_enum->srv->priv->targets)
@@ -560,10 +565,10 @@ g_network_service_address_enumerator_next_async (GSocketAddressEnumerator  *enum
                                        srv_enum->srv->priv->domain,
                                        cancellable,
                                        next_async_resolved_targets,
-                                       task);
+                                       srv_enum);
     }
   else
-    next_async_have_targets (task);
+    next_async_have_targets (srv_enum);
 }
 
 static void
@@ -571,8 +576,7 @@ next_async_resolved_targets (GObject      *source_object,
                              GAsyncResult *result,
                              gpointer      user_data)
 {
-  GTask *task = user_data;
-  GNetworkServiceAddressEnumerator *srv_enum = g_task_get_source_object (task);
+  GNetworkServiceAddressEnumerator *srv_enum = user_data;
   GError *error = NULL;
   GList *targets;
 
@@ -589,21 +593,23 @@ next_async_resolved_targets (GObject      *source_object,
 
   if (error)
     {
-      g_task_return_error (task, error);
-      g_object_unref (task);
+      GSimpleAsyncResult *simple = srv_enum->result;
+
+      srv_enum->result = NULL;
+      g_simple_async_result_take_error (simple, error);
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
     }
   else
     {
       srv_enum->t = srv_enum->srv->priv->targets = targets;
-      next_async_have_targets (task);
+      next_async_have_targets (srv_enum);
     }
 }
 
 static void
-next_async_have_targets (GTask *task)
+next_async_have_targets (GNetworkServiceAddressEnumerator *srv_enum)
 {
-  GNetworkServiceAddressEnumerator *srv_enum = g_task_get_source_object (task);
-
   /* Delegate to GNetworkAddress */
   if (srv_enum->addr_enum == NULL && srv_enum->t)
     {
@@ -625,21 +631,23 @@ next_async_have_targets (GTask *task)
   if (srv_enum->addr_enum)
     {
       g_socket_address_enumerator_next_async (srv_enum->addr_enum,
-                                              g_task_get_cancellable (task),
+                                              srv_enum->cancellable,
                                               next_async_have_address,
-                                              task);
+                                              srv_enum);
     }
   else
     {
+      GSimpleAsyncResult *simple = srv_enum->result;
+      srv_enum->result = NULL;
+
       if (srv_enum->error)
         {
-          g_task_return_error (task, srv_enum->error);
+          g_simple_async_result_take_error (simple, srv_enum->error);
           srv_enum->error = NULL;
         }
-      else
-        g_task_return_pointer (task, NULL, NULL);
 
-      g_object_unref (task);
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
     }
 }
 
@@ -648,8 +656,7 @@ next_async_have_address (GObject      *source_object,
                          GAsyncResult *result,
                          gpointer      user_data)
 {
-  GTask *task = user_data;
-  GNetworkServiceAddressEnumerator *srv_enum = g_task_get_source_object (task);
+  GNetworkServiceAddressEnumerator *srv_enum = user_data;
   GSocketAddress *address;
   GError *error = NULL;
   
@@ -670,12 +677,20 @@ next_async_have_address (GObject      *source_object,
       g_object_unref (srv_enum->addr_enum);
       srv_enum->addr_enum = NULL;
 
-      next_async_have_targets (task);
+      next_async_have_targets (srv_enum);
     }
   else
     {
-      g_task_return_pointer (task, address, g_object_unref);
-      g_object_unref (task);
+      GSimpleAsyncResult *simple = srv_enum->result;
+
+      srv_enum->result = NULL;
+
+      if (address)
+        g_simple_async_result_set_op_res_gpointer (simple,
+                                                   address, g_object_unref);
+
+      g_simple_async_result_complete (simple);
+      g_object_unref (simple);
     }
 }
 
@@ -684,7 +699,14 @@ g_network_service_address_enumerator_next_finish (GSocketAddressEnumerator  *enu
                                                   GAsyncResult              *result,
                                                   GError                   **error)
 {
-  return g_task_propagate_pointer (G_TASK (result), error);
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+  GSocketAddress *sockaddr;
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+
+  sockaddr = g_simple_async_result_get_op_res_gpointer (simple);
+  return sockaddr ? g_object_ref (sockaddr) : NULL;
 }
 
 static void
@@ -707,6 +729,12 @@ g_network_service_address_enumerator_finalize (GObject *object)
   if (srv_enum->resolver)
     g_object_unref (srv_enum->resolver);
 
+  if (srv_enum->result)
+    g_object_unref (srv_enum->result);
+
+  if (srv_enum->cancellable)
+    g_object_unref (srv_enum->cancellable);
+
   if (srv_enum->error)
     g_error_free (srv_enum->error);
 
@@ -733,7 +761,7 @@ g_network_service_connectable_enumerate (GSocketConnectable *connectable)
   GNetworkServiceAddressEnumerator *srv_enum;
 
   srv_enum = g_object_new (G_TYPE_NETWORK_SERVICE_ADDRESS_ENUMERATOR, NULL);
-  srv_enum->srv = g_object_ref (G_NETWORK_SERVICE (connectable));
+  srv_enum->srv = g_object_ref (connectable);
   srv_enum->resolver = g_resolver_get_default ();
   srv_enum->use_proxy = FALSE;
 
@@ -752,15 +780,3 @@ g_network_service_connectable_proxy_enumerate (GSocketConnectable *connectable)
 
   return addr_enum;
 }
-
-static gchar *
-g_network_service_connectable_to_string (GSocketConnectable *connectable)
-{
-  GNetworkService *service;
-
-  service = G_NETWORK_SERVICE (connectable);
-
-  return g_strdup_printf ("(%s, %s, %s, %s)", service->priv->service,
-                          service->priv->protocol, service->priv->domain,
-                          service->priv->scheme);
-}
