@@ -41,7 +41,7 @@ typedef struct _GSettingsBackendWatch   GSettingsBackendWatch;
 struct _GSettingsBackendPrivate
 {
   GSettingsBackendWatch *watches;
-  GStaticMutex lock;
+  GMutex lock;
 };
 
 /* For g_settings_backend_sync_default(), we only want to actually do
@@ -156,7 +156,7 @@ g_settings_backend_watch_weak_notify (gpointer  data,
   GSettingsBackendWatch **ptr;
 
   /* search and remove */
-  g_static_mutex_lock (&backend->priv->lock);
+  g_mutex_lock (&backend->priv->lock);
   for (ptr = &backend->priv->watches; *ptr; ptr = &(*ptr)->next)
     if ((*ptr)->target == where_the_object_was)
       {
@@ -165,7 +165,7 @@ g_settings_backend_watch_weak_notify (gpointer  data,
         *ptr = tmp->next;
         g_slice_free (GSettingsBackendWatch, tmp);
 
-        g_static_mutex_unlock (&backend->priv->lock);
+        g_mutex_unlock (&backend->priv->lock);
         return;
       }
 
@@ -190,7 +190,7 @@ g_settings_backend_watch_weak_notify (gpointer  data,
  * that appears as an argument to some of the callbacks, you *must* have
  * @context as %NULL.  Otherwise, you are subject to cross-thread
  * dispatching and whatever owned @origin_tag at the time that the event
- * occured may no longer own it.  This is a problem if you consider that
+ * occurred may no longer own it.  This is a problem if you consider that
  * you may now be the new owner of that address and mistakenly think
  * that the event in question originated from yourself.
  *
@@ -246,10 +246,10 @@ g_settings_backend_watch (GSettingsBackend              *backend,
   g_object_weak_ref (target, g_settings_backend_watch_weak_notify, backend);
 
   /* linked list prepend */
-  g_static_mutex_lock (&backend->priv->lock);
+  g_mutex_lock (&backend->priv->lock);
   watch->next = backend->priv->watches;
   backend->priv->watches = watch;
-  g_static_mutex_unlock (&backend->priv->lock);
+  g_mutex_unlock (&backend->priv->lock);
 }
 
 void
@@ -323,11 +323,11 @@ g_settings_backend_dispatch_signal (GSettingsBackend *backend,
    * count dropping -- so just add a reference to everything in the
    * suffix.
    */
-  g_static_mutex_lock (&backend->priv->lock);
+  g_mutex_lock (&backend->priv->lock);
   suffix = backend->priv->watches;
   for (watch = suffix; watch; watch = watch->next)
     g_object_ref (watch->target);
-  g_static_mutex_unlock (&backend->priv->lock);
+  g_mutex_unlock (&backend->priv->lock);
 
   /* The suffix is now immutable, so this is safe. */
   for (watch = suffix; watch; watch = next)
@@ -378,7 +378,7 @@ g_settings_backend_dispatch_signal (GSettingsBackend *backend,
  * dispatching the signal later.
  *
  * The implementation may call this function at any other time it likes
- * in response to other events (such as changes occuring outside of the
+ * in response to other events (such as changes occurring outside of the
  * program).  These calls may originate from a mainloop or may originate
  * in response to any other action (including from calls to
  * g_settings_backend_write()).
@@ -676,7 +676,6 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
                                  GTree            *tree,
                                  gpointer          origin_tag)
 {
-  GSettingsBackendWatch *watch;
   const gchar **keys;
   gchar *path;
 
@@ -696,10 +695,7 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
   }
 #endif
 
-  for (watch = backend->priv->watches; watch; watch = watch->next)
-    watch->vtable->keys_changed (watch->target, backend,
-                                 path, keys, origin_tag);
-
+  g_settings_backend_keys_changed (backend, path, keys, origin_tag);
   g_free (path);
   g_free (keys);
 }
@@ -710,7 +706,6 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
  * @key: the key to read
  * @expected_type: a #GVariantType
  * @default_value: if the default value should be returned
- * @returns: the value that was read, or %NULL
  *
  * Reads a key. This call will never block.
  *
@@ -724,6 +719,8 @@ g_settings_backend_changed_tree (GSettingsBackend *backend,
  * If @default_value is %TRUE then this gets the default value from the
  * backend (ie: the one that the backend would contain if
  * g_settings_reset() were called).
+ *
+ * Returns: the value that was read, or %NULL
  */
 GVariant *
 g_settings_backend_read (GSettingsBackend   *backend,
@@ -751,7 +748,6 @@ g_settings_backend_read (GSettingsBackend   *backend,
  * @key: the name of the key
  * @value: a #GVariant value to write to this key
  * @origin_tag: the origin tag
- * @returns: %TRUE if the write succeeded, %FALSE if the key was not writable
  *
  * Writes exactly one key.
  *
@@ -764,6 +760,8 @@ g_settings_backend_read (GSettingsBackend   *backend,
  * to emit a second "changed" signal (either during this call, or later)
  * to indicate that the affected keys have suddenly "changed back" to their
  * old values.
+ *
+ * Returns: %TRUE if the write succeeded, %FALSE if the key was not writable
  */
 gboolean
 g_settings_backend_write (GSettingsBackend *backend,
@@ -830,7 +828,6 @@ g_settings_backend_reset (GSettingsBackend *backend,
  * g_settings_backend_get_writable:
  * @backend: a #GSettingsBackend implementation
  * @key: the name of a key
- * @returns: %TRUE if the key is writable
  *
  * Finds out if a key is available for writing to.  This is the
  * interface through which 'lockdown' is implemented.  Locked down
@@ -838,6 +835,8 @@ g_settings_backend_reset (GSettingsBackend *backend,
  *
  * You should not write to locked-down keys, but if you do, the
  * implementation will deal with it.
+ *
+ * Returns: %TRUE if the key is writable
  */
 gboolean
 g_settings_backend_get_writable (GSettingsBackend *backend,
@@ -883,7 +882,7 @@ g_settings_backend_finalize (GObject *object)
 {
   GSettingsBackend *backend = G_SETTINGS_BACKEND (object);
 
-  g_static_mutex_unlock (&backend->priv->lock);
+  g_mutex_clear (&backend->priv->lock);
 
   G_OBJECT_CLASS (g_settings_backend_parent_class)
     ->finalize (object);
@@ -901,7 +900,7 @@ g_settings_backend_init (GSettingsBackend *backend)
   backend->priv = G_TYPE_INSTANCE_GET_PRIVATE (backend,
                                                G_TYPE_SETTINGS_BACKEND,
                                                GSettingsBackendPrivate);
-  g_static_mutex_init (&backend->priv->lock);
+  g_mutex_init (&backend->priv->lock);
 }
 
 static void
@@ -926,11 +925,12 @@ g_settings_backend_variant_unref0 (gpointer data)
 
 /*< private >
  * g_settings_backend_create_tree:
- * @returns: a new #GTree
  *
  * This is a convenience function for creating a tree that is compatible
  * with g_settings_backend_write().  It merely calls g_tree_new_full()
  * with strcmp(), g_free() and g_variant_unref().
+ *
+ * Returns: a new #GTree
  */
 GTree *
 g_settings_backend_create_tree (void)
@@ -939,9 +939,24 @@ g_settings_backend_create_tree (void)
                           g_free, g_settings_backend_variant_unref0);
 }
 
+static gboolean
+g_settings_backend_verify (gpointer impl)
+{
+  GSettingsBackend *backend = impl;
+
+  if (strcmp (G_OBJECT_TYPE_NAME (backend), "GMemorySettingsBackend") == 0 &&
+      g_strcmp0 (g_getenv ("GSETTINGS_BACKEND"), "memory") != 0)
+    {
+      g_message ("Using the 'memory' GSettings backend.  Your settings "
+		 "will not be saved or shared with other applications.");
+    }
+
+  g_settings_has_backend = TRUE;
+  return TRUE;
+}
+
 /**
  * g_settings_backend_get_default:
- * @returns: (transfer full): the default #GSettingsBackend
  *
  * Returns the default #GSettingsBackend. It is possible to override
  * the default by setting the <envar>GSETTINGS_BACKEND</envar>
@@ -949,71 +964,33 @@ g_settings_backend_create_tree (void)
  *
  * The user gets a reference to the backend.
  *
+ * Returns: (transfer full): the default #GSettingsBackend
+ *
  * Since: 2.28
  */
 GSettingsBackend *
 g_settings_backend_get_default (void)
 {
-  static gsize backend;
+  GSettingsBackend *backend;
 
-  if (g_once_init_enter (&backend))
-    {
-      GSettingsBackend *instance;
-      GIOExtensionPoint *point;
-      GIOExtension *extension;
-      GType extension_type;
-      GList *extensions;
-      const gchar *env;
-
-      _g_io_modules_ensure_loaded ();
-
-      point = g_io_extension_point_lookup (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME);
-      extension = NULL;
-
-      if ((env = getenv ("GSETTINGS_BACKEND")))
-        {
-          extension = g_io_extension_point_get_extension_by_name (point, env);
-
-          if (extension == NULL)
-            g_warning ("Can't find GSettings backend '%s' given in "
-                       "GSETTINGS_BACKEND environment variable", env);
-        }
-
-      if (extension == NULL)
-        {
-          extensions = g_io_extension_point_get_extensions (point);
-
-          if (extensions == NULL)
-            g_error ("No GSettingsBackend implementations exist.");
-
-          extension = extensions->data;
-
-          if (strcmp (g_io_extension_get_name (extension), "memory") == 0)
-            g_message ("Using the 'memory' GSettings backend.  Your settings "
-                       "will not be saved or shared with other applications.");
-        }
-
-      extension_type = g_io_extension_get_type (extension);
-      instance = g_object_new (extension_type, NULL);
-      g_settings_has_backend = TRUE;
-
-      g_once_init_leave (&backend, (gsize) instance);
-    }
-
-  return g_object_ref ((void *) backend);
+  backend = _g_io_module_get_default (G_SETTINGS_BACKEND_EXTENSION_POINT_NAME,
+				      "GSETTINGS_BACKEND",
+				      g_settings_backend_verify);
+  return g_object_ref (backend);
 }
 
 /*< private >
  * g_settings_backend_get_permission:
  * @backend: a #GSettingsBackend
  * @path: a path
- * @returns: a non-%NULL #GPermission. Free with g_object_unref()
  *
  * Gets the permission object associated with writing to keys below
  * @path on @backend.
  *
  * If this is not implemented in the backend, then a %TRUE
  * #GSimplePermission is returned.
+ *
+ * Returns: a non-%NULL #GPermission. Free with g_object_unref()
  */
 GPermission *
 g_settings_backend_get_permission (GSettingsBackend *backend,
