@@ -7,7 +7,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Alexander Larsson <alexl@redhat.com>
  *         David Zeuthen <davidz@redhat.com>
@@ -39,13 +41,13 @@
  * on the computer. In other words, what a file selector or file manager
  * would show in a sidebar. 
  *
- * #GVolumeMonitor is not
- * [thread-default-context aware][g-main-context-push-thread-default],
- * and so should not be used other than from the main thread, with no
- * thread-default-context active.
+ * #GVolumeMonitor is not <link
+ * linkend="g-main-context-push-thread-default">thread-default-context
+ * aware</link>, and so should not be used other than from the main
+ * thread, with no thread-default-context active.
  **/
 
-G_DEFINE_TYPE (GVolumeMonitor, g_volume_monitor, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GVolumeMonitor, g_volume_monitor, G_TYPE_OBJECT);
 
 enum {
   VOLUME_ADDED,
@@ -159,10 +161,7 @@ g_volume_monitor_class_init (GVolumeMonitorClass *klass)
    * @volume_monitor: The volume monitor emitting the signal.
    * @mount: a #GMount that is being unmounted.
    *
-   * May be emitted when a mount is about to be removed.
-   *
-   * This signal depends on the backend and is only emitted if
-   * GIO was used to unmount.
+   * Emitted when a mount is about to be removed.
    **/ 
   signals[MOUNT_PRE_UNMOUNT] = g_signal_new (I_("mount-pre-unmount"),
                                              G_TYPE_VOLUME_MONITOR,
