@@ -2,12 +2,10 @@
  *
  * Copyright 2011 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "gio.h"
@@ -106,8 +106,8 @@ GInetAddressMask *ip4_default, *ip6_default;
 
 static void
 notify_handler (GObject    *object,
-                GParamSpec *pspec,
-                gpointer    user_data)
+		GParamSpec *pspec,
+		gpointer    user_data)
 {
   gboolean *emitted = user_data;
 
@@ -116,8 +116,8 @@ notify_handler (GObject    *object,
 
 static void
 network_changed_handler (GNetworkMonitor *monitor,
-                         gboolean         available,
-                         gpointer         user_data)
+			 gboolean         available,
+			 gpointer         user_data)
 {
   gboolean *emitted = user_data;
 
@@ -126,19 +126,19 @@ network_changed_handler (GNetworkMonitor *monitor,
 
 static void
 assert_signals (GNetworkMonitor *monitor,
-                gboolean         should_emit_notify,
-                gboolean         should_emit_network_changed,
-                gboolean         expected_network_available)
+		gboolean         should_emit_notify,
+		gboolean         should_emit_network_changed,
+		gboolean         expected_network_available)
 {
   gboolean emitted_notify = FALSE, emitted_network_changed = FALSE;
   guint h1, h2;
 
   h1 = g_signal_connect (monitor, "notify::network-available",
-                         G_CALLBACK (notify_handler),
-                         &emitted_notify);
+			 G_CALLBACK (notify_handler),
+			 &emitted_notify);
   h2 = g_signal_connect (monitor, "network-changed",
-                         G_CALLBACK (network_changed_handler),
-                         &emitted_network_changed);
+			 G_CALLBACK (network_changed_handler),
+			 &emitted_network_changed);
 
   g_main_context_iteration (NULL, FALSE);
 
@@ -151,100 +151,41 @@ assert_signals (GNetworkMonitor *monitor,
   g_assert (g_network_monitor_get_network_available (monitor) == expected_network_available);
 }
 
-typedef struct {
-  GNetworkMonitor *monitor;
-  GMainLoop       *loop;
-  GSocketAddress  *sockaddr;
-  gboolean         should_be_reachable;
-} CanReachData;
-
-static void
-reach_cb (GObject      *source,
-          GAsyncResult *res,
-          gpointer      user_data)
-{
-  GError *error = NULL;
-  gboolean reachable;
-  CanReachData *data = user_data;
-
-  reachable = g_network_monitor_can_reach_finish (data->monitor, res, &error);
-
-  if (data->should_be_reachable)
-    g_assert_no_error (error);
-  else
-    {
-      g_assert (error != NULL);
-      g_clear_error (&error);
-    }
-  g_assert (reachable == data->should_be_reachable);
-
-  g_main_loop_quit (data->loop);
-}
-
-static gboolean
-test_reach_async (gpointer user_data)
-{
-  CanReachData *data = user_data;
-
-  g_network_monitor_can_reach_async (data->monitor,
-                                     G_SOCKET_CONNECTABLE (data->sockaddr),
-                                     NULL,
-                                     reach_cb,
-                                     data);
-  return G_SOURCE_REMOVE;
-}
-
 static void
 run_tests (GNetworkMonitor *monitor,
-           TestAddress     *addresses,
-           gboolean         should_be_reachable)
+	   TestAddress     *addresses,
+	   gboolean         should_be_reachable)
 {
   GError *error = NULL;
   int i;
   gboolean reachable;
   GSocketAddress *sockaddr;
-  CanReachData data;
-
-  data.monitor = monitor;
-  data.loop = g_main_loop_new (NULL, FALSE);
 
   for (i = 0; addresses[i].address; i++)
     {
       sockaddr = g_inet_socket_address_new (addresses[i].address, 0);
       reachable = g_network_monitor_can_reach (monitor,
-                                               G_SOCKET_CONNECTABLE (sockaddr),
-                                               NULL, &error);
-      data.sockaddr = sockaddr;
-      data.should_be_reachable = should_be_reachable;
-
-      g_idle_add (test_reach_async, &data);
-      g_main_loop_run (data.loop);
-
+					       G_SOCKET_CONNECTABLE (sockaddr),
+					       NULL, &error);
       g_object_unref (sockaddr);
       g_assert_cmpint (reachable, ==, should_be_reachable);
       if (should_be_reachable)
-        g_assert_no_error (error);
+	g_assert_no_error (error);
       else
-        {
-          g_assert (error != NULL);
-          g_clear_error (&error);
-        }
+	{
+	  g_assert (error != NULL);
+	  g_clear_error (&error);
+	}
     }
-
-  g_main_loop_unref (data.loop);
 }
 
 static void
 test_default (void)
 {
-  GNetworkMonitor *monitor, *m;
+  GNetworkMonitor *monitor;
   GError *error = NULL;
 
-  m = g_network_monitor_get_default ();
-  g_assert (G_IS_NETWORK_MONITOR (m));
-
-  monitor = g_object_new (G_TYPE_NETWORK_MONITOR_BASE, NULL);
-  g_initable_init (G_INITABLE (monitor), NULL, &error);
+  monitor = g_initable_new (G_TYPE_NETWORK_MONITOR_BASE, NULL, &error, NULL);
   g_assert_no_error (error);
 
   /* In the default configuration, all addresses are reachable */
@@ -256,8 +197,6 @@ test_default (void)
   run_tests (monitor, unmatched, TRUE);
 
   assert_signals (monitor, FALSE, FALSE, TRUE);
-
-  g_object_unref (monitor);
 }
 
 static void
@@ -271,10 +210,10 @@ test_remove_default (void)
   assert_signals (monitor, FALSE, FALSE, TRUE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip4_default);
+					 ip4_default);
   assert_signals (monitor, FALSE, TRUE, TRUE);
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip6_default);
+					 ip6_default);
   assert_signals (monitor, TRUE, TRUE, FALSE);
 
   /* Now nothing should be reachable */
@@ -284,8 +223,6 @@ test_remove_default (void)
   run_tests (monitor, netlocal6.addresses, FALSE);
   run_tests (monitor, netfe80.addresses, FALSE);
   run_tests (monitor, unmatched, FALSE);
-
-  g_object_unref (monitor);
 }
 
 static void
@@ -299,16 +236,16 @@ test_add_networks (void)
   assert_signals (monitor, FALSE, FALSE, TRUE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip4_default);
+					 ip4_default);
   assert_signals (monitor, FALSE, TRUE, TRUE);
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip6_default);
+					 ip6_default);
   assert_signals (monitor, TRUE, TRUE, FALSE);
 
   /* Now add the masks one by one */
 
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net127.mask);
+				      net127.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
 
   run_tests (monitor, net127.addresses, TRUE);
@@ -319,7 +256,7 @@ test_add_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net10.mask);
+				      net10.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, TRUE);
   run_tests (monitor, net10.addresses, TRUE);
@@ -329,7 +266,7 @@ test_add_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net192.mask);
+				      net192.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, TRUE);
   run_tests (monitor, net10.addresses, TRUE);
@@ -339,7 +276,7 @@ test_add_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      netlocal6.mask);
+				      netlocal6.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, TRUE);
   run_tests (monitor, net10.addresses, TRUE);
@@ -349,7 +286,7 @@ test_add_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      netfe80.mask);
+				      netfe80.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, TRUE);
   run_tests (monitor, net10.addresses, TRUE);
@@ -357,8 +294,6 @@ test_add_networks (void)
   run_tests (monitor, netlocal6.addresses, TRUE);
   run_tests (monitor, netfe80.addresses, TRUE);
   run_tests (monitor, unmatched, FALSE);
-
-  g_object_unref (monitor);
 }
 
 static void
@@ -372,27 +307,27 @@ test_remove_networks (void)
   assert_signals (monitor, FALSE, FALSE, TRUE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip4_default);
+					 ip4_default);
   assert_signals (monitor, FALSE, TRUE, TRUE);
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         ip6_default);
+					 ip6_default);
   assert_signals (monitor, TRUE, TRUE, FALSE);
 
   /* First add them */
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net127.mask);
+				      net127.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net10.mask);
+				      net10.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      net192.mask);
+				      net192.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      netlocal6.mask);
+				      netlocal6.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   g_network_monitor_base_add_network (G_NETWORK_MONITOR_BASE (monitor),
-                                      netfe80.mask);
+				      netfe80.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
 
   run_tests (monitor, net127.addresses, TRUE);
@@ -404,7 +339,7 @@ test_remove_networks (void)
 
   /* Now remove them one by one */
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         net127.mask);
+					 net127.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, FALSE);
   run_tests (monitor, net10.addresses, TRUE);
@@ -414,7 +349,7 @@ test_remove_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         net10.mask);
+					 net10.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, FALSE);
   run_tests (monitor, net10.addresses, FALSE);
@@ -424,7 +359,7 @@ test_remove_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         net192.mask);
+					 net192.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, FALSE);
   run_tests (monitor, net10.addresses, FALSE);
@@ -434,7 +369,7 @@ test_remove_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         netlocal6.mask);
+					 netlocal6.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, FALSE);
   run_tests (monitor, net10.addresses, FALSE);
@@ -444,7 +379,7 @@ test_remove_networks (void)
   run_tests (monitor, unmatched, FALSE);
 
   g_network_monitor_base_remove_network (G_NETWORK_MONITOR_BASE (monitor),
-                                         netfe80.mask);
+					 netfe80.mask);
   assert_signals (monitor, FALSE, TRUE, FALSE);
   run_tests (monitor, net127.addresses, FALSE);
   run_tests (monitor, net10.addresses, FALSE);
@@ -452,8 +387,6 @@ test_remove_networks (void)
   run_tests (monitor, netlocal6.addresses, FALSE);
   run_tests (monitor, netfe80.addresses, FALSE);
   run_tests (monitor, unmatched, FALSE);
-
-  g_object_unref (monitor);
 }
 
 
@@ -470,46 +403,20 @@ init_test (TestMask *test)
     {
       test->addresses[i].address = g_inet_address_new_from_string (test->addresses[i].string);
       if (strchr (test->addresses[i].string, ':'))
-        g_assert_cmpint (g_inet_address_get_family (test->addresses[i].address), ==, G_SOCKET_FAMILY_IPV6);
+	g_assert_cmpint (g_inet_address_get_family (test->addresses[i].address), ==, G_SOCKET_FAMILY_IPV6);
       else
-        g_assert_cmpint (g_inet_address_get_family (test->addresses[i].address), ==, G_SOCKET_FAMILY_IPV4);
+	g_assert_cmpint (g_inet_address_get_family (test->addresses[i].address), ==, G_SOCKET_FAMILY_IPV4);
     }
 }
 
-static void
-cleanup_test (TestMask *test)
-{
-  int i;
-
-  g_object_unref (test->mask);
-  for (i = 0; test->addresses[i].string; i++)
-    g_object_unref (test->addresses[i].address);
-}
-
 static void
 watch_network_changed (GNetworkMonitor *monitor,
-                       gboolean         available,
-                       gpointer         user_data)
+		       gboolean         available,
+		       gpointer         user_data)
 {
   g_print ("Network is %s\n", available ? "up" : "down");
 }
 
-static void
-watch_connectivity_changed (GNetworkMonitor *monitor,
-			    GParamSpec      *pspec,
-			    gpointer         user_data)
-{
-  g_print ("Connectivity is %d\n", g_network_monitor_get_connectivity (monitor));
-}
-
-static void
-watch_metered_changed (GNetworkMonitor *monitor,
-                       GParamSpec      *pspec,
-                       gpointer         user_data)
-{
-  g_print ("Metered is %d\n", g_network_monitor_get_network_metered (monitor));
-}
-
 static void
 do_watch_network (void)
 {
@@ -519,14 +426,8 @@ do_watch_network (void)
   g_print ("Monitoring via %s\n", g_type_name_from_instance ((GTypeInstance *) monitor));
 
   g_signal_connect (monitor, "network-changed",
-                    G_CALLBACK (watch_network_changed), NULL);
-  g_signal_connect (monitor, "notify::connectivity",
-                    G_CALLBACK (watch_connectivity_changed), NULL);
-  g_signal_connect (monitor, "notify::network-metered",
-                    G_CALLBACK (watch_metered_changed), NULL);
+		    G_CALLBACK (watch_network_changed), NULL);
   watch_network_changed (monitor, g_network_monitor_get_network_available (monitor), NULL);
-  watch_connectivity_changed (monitor, NULL, NULL);
-  watch_metered_changed (monitor, NULL, NULL);
 
   loop = g_main_loop_new (NULL, FALSE);
   g_main_loop_run (loop);
@@ -535,7 +436,7 @@ do_watch_network (void)
 int
 main (int argc, char **argv)
 {
-  int ret;
+  g_type_init ();
 
   if (argc == 2 && !strcmp (argv[1], "--watch"))
     {
@@ -545,16 +446,6 @@ main (int argc, char **argv)
 
   g_test_init (&argc, &argv, NULL);
 
-  /* GNetworkMonitor will resolve addresses through a proxy if one is set and a
-   * GIO module is available to handle it. In these tests we deliberately
-   * change the idea of a reachable network to exclude the proxy, which will
-   * lead to negative results. We're not trying to test the proxy-resolving
-   * functionality (that would be for e.g. glib-networking's testsuite), so
-   * let's just use the dummy proxy resolver, which always pretends the
-   * passed-in URL is directly resolvable.
-   */
-  g_setenv ("GIO_USE_PROXY_RESOLVER", "dummy", TRUE);
-
   init_test (&net127);
   init_test (&net10);
   init_test (&net192);
@@ -568,15 +459,5 @@ main (int argc, char **argv)
   g_test_add_func ("/network-monitor/add_networks", test_add_networks);
   g_test_add_func ("/network-monitor/remove_networks", test_remove_networks);
 
-  ret = g_test_run ();
-
-  cleanup_test (&net127);
-  cleanup_test (&net10);
-  cleanup_test (&net192);
-  cleanup_test (&netlocal6);
-  cleanup_test (&netfe80);
-  g_object_unref (ip4_default);
-  g_object_unref (ip6_default);
-
-  return ret;
+  return g_test_run ();
 }
