@@ -5,12 +5,10 @@
  * Copyright 1998 Sebastian Wilhelmi; University of Karlsruhe
  *                Owen Taylor
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,15 +16,15 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 /* we know we are deprecated here, no need for warnings */
-#ifndef GLIB_DISABLE_DEPRECATION_WARNINGS
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
-#endif
 
 #include "gmessages.h"
 #include "gslice.h"
@@ -41,19 +39,28 @@
 /* {{{1 Documentation */
 
 /**
- * GThreadPriority: (skip):
+ * SECTION:threads-deprecated
+ * @title: Deprecated thread API
+ * @short_description: old thread APIs (for reference only)
+ * @see_also: #GThread
+ *
+ * These APIs are deprecated.  You should not use them in new code.
+ * This section remains only to assist with understanding code that was
+ * written to use these APIs at some point in the past.
+ **/
+
+/**
+ * GThreadPriority:
  * @G_THREAD_PRIORITY_LOW: a priority lower than normal
  * @G_THREAD_PRIORITY_NORMAL: the default priority
  * @G_THREAD_PRIORITY_HIGH: a priority higher than normal
  * @G_THREAD_PRIORITY_URGENT: the highest priority
  *
- * Thread priorities.
- *
  * Deprecated:2.32: Thread priorities no longer have any effect.
  */
 
 /**
- * GThreadFunctions: (skip):
+ * GThreadFunctions:
  * @mutex_new: virtual function pointer for g_mutex_new()
  * @mutex_lock: virtual function pointer for g_mutex_lock()
  * @mutex_trylock: virtual function pointer for g_mutex_trylock()
@@ -83,20 +90,20 @@
  */
 
 /**
- * G_THREADS_IMPL_POSIX: (skip):
+ * G_THREADS_IMPL_POSIX:
  *
  * This macro is defined if POSIX style threads are used.
  *
  * Deprecated:2.32:POSIX threads are in use on all non-Windows systems.
- *   Use %G_OS_WIN32 to detect Windows.
+ *                 Use G_OS_WIN32 to detect Windows.
  */
 
 /**
- * G_THREADS_IMPL_WIN32: (skip):
+ * G_THREADS_IMPL_WIN32:
  *
  * This macro is defined if Windows style threads are used.
  *
- * Deprecated:2.32:Use %G_OS_WIN32 to detect Windows.
+ * Deprecated:2.32:Use G_OS_WIN32 to detect Windows.
  */
 
 
@@ -135,7 +142,7 @@ GThreadFunctions g_thread_functions_for_glib_use =
 static guint64
 gettime (void)
 {
-  return (guint64) g_get_monotonic_time () * 1000;
+  return g_get_monotonic_time () * 1000;
 }
 
 guint64 (*g_thread_gettime) (void) = gettime;
@@ -144,7 +151,7 @@ guint64 (*g_thread_gettime) (void) = gettime;
 gboolean         g_threads_got_initialized = TRUE;
 
 /**
- * g_thread_init: (skip):
+ * g_thread_init:
  * @vtable: a function table of type #GThreadFunctions, that provides
  *     the entry points to the thread system to be used. Since 2.32,
  *     this parameter is ignored and should always be %NULL
@@ -158,29 +165,19 @@ gboolean         g_threads_got_initialized = TRUE;
  * Since version 2.32, GLib does not support custom thread implementations
  * anymore and the @vtable parameter is ignored and you should pass %NULL.
  *
- * ::: note
- *     g_thread_init() must not be called directly or indirectly in a
- *     callback from GLib. Also no mutexes may be currently locked
- *     while calling g_thread_init().
+ * <note><para>g_thread_init() must not be called directly or indirectly
+ * in a callback from GLib. Also no mutexes may be currently locked while
+ * calling g_thread_init().</para></note>
  *
- * ::: note
- *     To use g_thread_init() in your program, you have to link with
- *     the libraries that the command `pkg-config --libs gthread-2.0`
- *     outputs. This is not the case for all the other thread-related
- *     functions of GLib. Those can be used without having to link
- *     with the thread libraries.
- *
- * Deprecated:2.32: This function is no longer necessary. The GLib
- *     threading system is automatically initialized at the start
- *     of your program.
+ * <note><para>To use g_thread_init() in your program, you have to link
+ * with the libraries that the command <command>pkg-config --libs
+ * gthread-2.0</command> outputs. This is not the case for all the
+ * other thread-related functions of GLib. Those can be used without
+ * having to link with the thread libraries.</para></note>
  */
 
 /**
- * g_thread_init_with_errorcheck_mutexes: (skip):
- */
-
-/**
- * g_thread_get_initialized: (skip):
+ * g_thread_get_initialized:
  *
  * Indicates if g_thread_init() has been called.
  *
@@ -195,7 +192,6 @@ g_thread_get_initialized (void)
 }
 
 /* We need this for ABI compatibility */
-GLIB_AVAILABLE_IN_ALL
 void g_thread_init_glib (void);
 void g_thread_init_glib (void) { }
 
@@ -211,7 +207,7 @@ G_LOCK_DEFINE_STATIC (g_thread);
 /* Misc. GThread functions {{{1 */
 
 /**
- * g_thread_set_priority: (skip):
+ * g_thread_set_priority:
  * @thread: a #GThread.
  * @priority: ignored
  *
@@ -226,8 +222,8 @@ g_thread_set_priority (GThread         *thread,
 }
 
 /**
- * g_thread_foreach: (skip):
- * @thread_func: (scope call): function to call for all #GThread structures
+ * g_thread_foreach:
+ * @thread_func: function to call for all #GThread structures
  * @user_data: second argument to @thread_func
  *
  * Call @thread_func on all #GThreads that have been
@@ -310,7 +306,7 @@ g_deprecated_thread_proxy (gpointer data)
 }
 
 /**
- * g_thread_create: (skip):
+ * g_thread_create:
  * @func: a function to execute in the new thread
  * @data: an argument to supply to the new thread
  * @joinable: should this thread be joinable?
@@ -343,7 +339,7 @@ g_thread_create (GThreadFunc   func,
 }
 
 /**
- * g_thread_create_full: (skip):
+ * g_thread_create_full:
  * @func: a function to execute in the new thread.
  * @data: an argument to supply to the new thread.
  * @stack_size: a stack size for the new thread.
@@ -351,11 +347,10 @@ g_thread_create (GThreadFunc   func,
  * @bound: ignored
  * @priority: ignored
  * @error: return location for error.
+ * @Returns: the new #GThread on success.
  *
  * This function creates a new thread.
  *
- * Returns: the new #GThread on success.
- *
  * Deprecated:2.32: The @bound and @priority arguments are now ignored.
  * Use g_thread_new().
  */
@@ -373,7 +368,7 @@ g_thread_create_full (GThreadFunc       func,
   thread = g_thread_new_internal (NULL, g_deprecated_thread_proxy,
                                   func, data, stack_size, error);
 
-  if (thread && !joinable)
+  if (!joinable)
     {
       thread->joinable = FALSE;
       g_thread_unref (thread);
@@ -392,7 +387,7 @@ g_once_init_enter_impl (volatile gsize *location)
 /* GStaticMutex {{{1 ------------------------------------------------------ */
 
 /**
- * GStaticMutex: (skip):
+ * GStaticMutex:
  *
  * A #GStaticMutex works like a #GMutex.
  *
@@ -402,8 +397,14 @@ g_once_init_enter_impl (volatile gsize *location)
  * as well, and GStaticMutex has been deprecated.
  *
  * Here is a version of our give_me_next_number() example using
- * a GStaticMutex:
- * |[
+ * a GStaticMutex.
+ *
+ * <example>
+ *  <title>
+ *   Using <structname>GStaticMutex</structname>
+ *   to simplify thread-safe programming
+ *  </title>
+ *  <programlisting>
  *   int
  *   give_me_next_number (void)
  *   {
@@ -411,19 +412,20 @@ g_once_init_enter_impl (volatile gsize *location)
  *     int ret_val;
  *     static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
  *
- *     g_static_mutex_lock (&mutex);
+ *     g_static_mutex_lock (&amp;mutex);
  *     ret_val = current_number = calc_next_number (current_number);
- *     g_static_mutex_unlock (&mutex);
+ *     g_static_mutex_unlock (&amp;mutex);
  *
  *     return ret_val;
  *   }
- * ]|
+ *  </programlisting>
+ * </example>
  *
  * Sometimes you would like to dynamically create a mutex. If you don't
  * want to require prior calling to g_thread_init(), because your code
  * should also be usable in non-threaded programs, you are not able to
  * use g_mutex_new() and thus #GMutex, as that requires a prior call to
- * g_thread_init(). In these cases you can also use a #GStaticMutex.
+ * g_thread_init(). In theses cases you can also use a #GStaticMutex.
  * It must be initialized with g_static_mutex_init() before using it
  * and freed with with g_static_mutex_free() when not needed anymore to
  * free up any allocated resources.
@@ -432,18 +434,19 @@ g_once_init_enter_impl (volatile gsize *location)
  * the following functions, as it is defined differently on different
  * platforms.
  *
- * All of the g_static_mutex_* functions apart from
- * g_static_mutex_get_mutex() can also be used even if g_thread_init()
- * has not yet been called. Then they do nothing, apart from
- * g_static_mutex_trylock() which does nothing but returning %TRUE.
+ * All of the <function>g_static_mutex_*</function> functions apart
+ * from <function>g_static_mutex_get_mutex</function> can also be used
+ * even if g_thread_init() has not yet been called. Then they do
+ * nothing, apart from <function>g_static_mutex_trylock</function>,
+ * which does nothing but returning %TRUE.
  *
- * All of the g_static_mutex_* functions are actually macros. Apart from
- * taking their addresses, you can however use them as if they were
- * functions.
- */
+ * <note><para>All of the <function>g_static_mutex_*</function>
+ * functions are actually macros. Apart from taking their addresses, you
+ * can however use them as if they were functions.</para></note>
+ **/
 
 /**
- * G_STATIC_MUTEX_INIT: (skip):
+ * G_STATIC_MUTEX_INIT:
  *
  * A #GStaticMutex must be initialized with this macro, before it can
  * be used. This macro can used be to initialize a variable, but it
@@ -456,11 +459,11 @@ g_once_init_enter_impl (volatile gsize *location)
  **/
 
 /**
- * g_static_mutex_init: (skip):
+ * g_static_mutex_init:
  * @mutex: a #GStaticMutex to be initialized.
  *
  * Initializes @mutex.
- * Alternatively you can initialize it with %G_STATIC_MUTEX_INIT.
+ * Alternatively you can initialize it with #G_STATIC_MUTEX_INIT.
  *
  * Deprecated: 2.32: Use g_mutex_init()
  */
@@ -492,15 +495,14 @@ g_static_mutex_init (GStaticMutex *mutex)
  */
 
 /**
- * g_static_mutex_get_mutex: (skip):
+ * g_static_mutex_get_mutex:
  * @mutex: a #GStaticMutex.
+ * @Returns: the #GMutex corresponding to @mutex.
  *
  * For some operations (like g_cond_wait()) you must have a #GMutex
  * instead of a #GStaticMutex. This function will return the
  * corresponding #GMutex for @mutex.
  *
- * Returns: the #GMutex corresponding to @mutex.
- *
  * Deprecated: 2.32: Just use a #GMutex
  */
 GMutex *
@@ -539,7 +541,7 @@ g_static_mutex_get_mutex_impl (GStaticMutex* mutex)
  */
 
 /**
- * g_static_mutex_lock: (skip):
+ * g_static_mutex_lock:
  * @mutex: a #GStaticMutex.
  *
  * Works like g_mutex_lock(), but for a #GStaticMutex.
@@ -548,18 +550,17 @@ g_static_mutex_get_mutex_impl (GStaticMutex* mutex)
  */
 
 /**
- * g_static_mutex_trylock: (skip):
+ * g_static_mutex_trylock:
  * @mutex: a #GStaticMutex.
+ * @Returns: %TRUE, if the #GStaticMutex could be locked.
  *
  * Works like g_mutex_trylock(), but for a #GStaticMutex.
  *
- * Returns: %TRUE, if the #GStaticMutex could be locked.
- *
  * Deprecated: 2.32: Use g_mutex_trylock()
  */
 
 /**
- * g_static_mutex_unlock: (skip):
+ * g_static_mutex_unlock:
  * @mutex: a #GStaticMutex.
  *
  * Works like g_mutex_unlock(), but for a #GStaticMutex.
@@ -568,7 +569,7 @@ g_static_mutex_get_mutex_impl (GStaticMutex* mutex)
  */
 
 /**
- * g_static_mutex_free: (skip):
+ * g_static_mutex_free:
  * @mutex: a #GStaticMutex to be freed.
  *
  * Releases all resources allocated to @mutex.
@@ -578,10 +579,10 @@ g_static_mutex_get_mutex_impl (GStaticMutex* mutex)
  * a #GStaticMutex as a member of a structure and the structure is
  * freed, you should also free the #GStaticMutex.
  *
- * Calling g_static_mutex_free() on a locked mutex may result in
- * undefined behaviour.
+ * <note><para>Calling g_static_mutex_free() on a locked mutex may
+ * result in undefined behaviour.</para></note>
  *
- * Deprecated: 2.32: Use g_mutex_clear()
+ * Deprecated: 2.32: Use g_mutex_free()
  */
 void
 g_static_mutex_free (GStaticMutex* mutex)
@@ -606,7 +607,7 @@ g_static_mutex_free (GStaticMutex* mutex)
 /* {{{1 GStaticRecMutex */
 
 /**
- * GStaticRecMutex: (skip):
+ * GStaticRecMutex:
  *
  * A #GStaticRecMutex works like a #GStaticMutex, but it can be locked
  * multiple times by one thread. If you enter it n times, you have to
@@ -621,14 +622,14 @@ g_static_mutex_free (GStaticMutex* mutex)
  * Even though #GStaticRecMutex is not opaque, it should only be used
  * with the following functions.
  *
- * All of the g_static_rec_mutex_* functions can be used even if
- * g_thread_init() has not been called. Then they do nothing, apart
- * from g_static_rec_mutex_trylock(), which does nothing but returning
- * %TRUE.
- */
+ * All of the <function>g_static_rec_mutex_*</function> functions can
+ * be used even if g_thread_init() has not been called. Then they do
+ * nothing, apart from <function>g_static_rec_mutex_trylock</function>,
+ * which does nothing but returning %TRUE.
+ **/
 
 /**
- * G_STATIC_REC_MUTEX_INIT: (skip):
+ * G_STATIC_REC_MUTEX_INIT:
  *
  * A #GStaticRecMutex must be initialized with this macro before it can
  * be used. This macro can used be to initialize a variable, but it
@@ -641,12 +642,12 @@ g_static_mutex_free (GStaticMutex* mutex)
  */
 
 /**
- * g_static_rec_mutex_init: (skip):
+ * g_static_rec_mutex_init:
  * @mutex: a #GStaticRecMutex to be initialized.
  *
  * A #GStaticRecMutex must be initialized with this function before it
  * can be used. Alternatively you can initialize it with
- * %G_STATIC_REC_MUTEX_INIT.
+ * #G_STATIC_REC_MUTEX_INIT.
  *
  * Deprecated: 2.32: Use g_rec_mutex_init()
  */
@@ -668,7 +669,7 @@ g_static_rec_mutex_get_rec_mutex_impl (GStaticRecMutex* mutex)
   if (!g_thread_supported ())
     return NULL;
 
-  result = (GRecMutex *) g_atomic_pointer_get (&mutex->mutex.mutex);
+  result = g_atomic_pointer_get (&mutex->mutex.mutex);
 
   if (!result)
     {
@@ -679,7 +680,7 @@ g_static_rec_mutex_get_rec_mutex_impl (GStaticRecMutex* mutex)
         {
           result = g_slice_new (GRecMutex);
           g_rec_mutex_init (result);
-          g_atomic_pointer_set (&mutex->mutex.mutex, (GMutex *) result);
+          g_atomic_pointer_set (&mutex->mutex.mutex, result);
         }
 
       G_UNLOCK (g_static_mutex);
@@ -689,7 +690,7 @@ g_static_rec_mutex_get_rec_mutex_impl (GStaticRecMutex* mutex)
 }
 
 /**
- * g_static_rec_mutex_lock: (skip):
+ * g_static_rec_mutex_lock:
  * @mutex: a #GStaticRecMutex to lock.
  *
  * Locks @mutex. If @mutex is already locked by another thread, the
@@ -709,8 +710,9 @@ g_static_rec_mutex_lock (GStaticRecMutex* mutex)
 }
 
 /**
- * g_static_rec_mutex_trylock: (skip):
+ * g_static_rec_mutex_trylock:
  * @mutex: a #GStaticRecMutex to lock.
+ * @Returns: %TRUE, if @mutex could be locked.
  *
  * Tries to lock @mutex. If @mutex is already locked by another thread,
  * it immediately returns %FALSE. Otherwise it locks @mutex and returns
@@ -718,8 +720,6 @@ g_static_rec_mutex_lock (GStaticRecMutex* mutex)
  * functions increases the depth of @mutex and immediately returns
  * %TRUE.
  *
- * Returns: %TRUE, if @mutex could be locked.
- *
  * Deprecated: 2.32: Use g_rec_mutex_trylock()
  */
 gboolean
@@ -738,7 +738,7 @@ g_static_rec_mutex_trylock (GStaticRecMutex* mutex)
 }
 
 /**
- * g_static_rec_mutex_unlock: (skip):
+ * g_static_rec_mutex_unlock:
  * @mutex: a #GStaticRecMutex to unlock.
  *
  * Unlocks @mutex. Another thread will be allowed to lock @mutex only
@@ -759,7 +759,7 @@ g_static_rec_mutex_unlock (GStaticRecMutex* mutex)
 }
 
 /**
- * g_static_rec_mutex_lock_full: (skip):
+ * g_static_rec_mutex_lock_full:
  * @mutex: a #GStaticRecMutex to lock.
  * @depth: number of times this mutex has to be unlocked to be
  *         completely unlocked.
@@ -783,8 +783,10 @@ g_static_rec_mutex_lock_full (GStaticRecMutex *mutex,
 }
 
 /**
- * g_static_rec_mutex_unlock_full: (skip):
+ * g_static_rec_mutex_unlock_full:
  * @mutex: a #GStaticRecMutex to completely unlock.
+ * @Returns: number of times @mutex has been locked by the current
+ *           thread.
  *
  * Completely unlocks @mutex. If another thread is blocked in a
  * g_static_rec_mutex_lock() call for @mutex, it will be woken and can
@@ -794,33 +796,27 @@ g_static_rec_mutex_lock_full (GStaticRecMutex *mutex,
  * g_static_rec_mutex_lock_full() with the depth returned by this
  * function.
  *
- * Returns: number of times @mutex has been locked by the current
- *          thread.
- *
  * Deprecated: 2.32: Use g_rec_mutex_unlock()
  */
 guint
 g_static_rec_mutex_unlock_full (GStaticRecMutex *mutex)
 {
   GRecMutex *rm;
-  guint depth;
-  guint i;
+  gint depth;
 
   rm = g_static_rec_mutex_get_rec_mutex_impl (mutex);
-
-  /* all access to mutex->depth done while still holding the lock */
   depth = mutex->depth;
-  i = mutex->depth;
-  mutex->depth = 0;
-
-  while (i--)
-    g_rec_mutex_unlock (rm);
+  while (mutex->depth)
+    {
+      mutex->depth--;
+      g_rec_mutex_unlock (rm);
+    }
 
   return depth;
 }
 
 /**
- * g_static_rec_mutex_free: (skip):
+ * g_static_rec_mutex_free:
  * @mutex: a #GStaticRecMutex to be freed.
  *
  * Releases all resources allocated to a #GStaticRecMutex.
@@ -849,16 +845,18 @@ g_static_rec_mutex_free (GStaticRecMutex *mutex)
 /* GStaticRWLock {{{1 ----------------------------------------------------- */
 
 /**
- * GStaticRWLock: (skip):
+ * GStaticRWLock:
  *
  * The #GStaticRWLock struct represents a read-write lock. A read-write
  * lock can be used for protecting data that some portions of code only
  * read from, while others also write. In such situations it is
  * desirable that several readers can read at once, whereas of course
- * only one writer may write at a time.
+ * only one writer may write at a time. Take a look at the following
+ * example:
  *
- * Take a look at the following example:
- * |[
+ * <example>
+ *  <title>An array with access functions</title>
+ *  <programlisting>
  *   GStaticRWLock rwlock = G_STATIC_RW_LOCK_INIT;
  *   GPtrArray *array;
  *
@@ -870,10 +868,10 @@ g_static_rec_mutex_free (GStaticRecMutex *mutex)
  *     if (!array)
  *       return NULL;
  *
- *     g_static_rw_lock_reader_lock (&rwlock);
- *     if (index < array->len)
+ *     g_static_rw_lock_reader_lock (&amp;rwlock);
+ *     if (index &lt; array->len)
  *       retval = g_ptr_array_index (array, index);
- *     g_static_rw_lock_reader_unlock (&rwlock);
+ *     g_static_rw_lock_reader_unlock (&amp;rwlock);
  *
  *     return retval;
  *   }
@@ -881,25 +879,27 @@ g_static_rec_mutex_free (GStaticRecMutex *mutex)
  *   void
  *   my_array_set (guint index, gpointer data)
  *   {
- *     g_static_rw_lock_writer_lock (&rwlock);
+ *     g_static_rw_lock_writer_lock (&amp;rwlock);
  *
  *     if (!array)
- *       array = g_ptr_array_new ();
+ *       array = g_ptr_array_new (<!-- -->);
  *
  *     if (index >= array->len)
- *       g_ptr_array_set_size (array, index + 1);
+ *       g_ptr_array_set_size (array, index+1);
  *     g_ptr_array_index (array, index) = data;
  *
- *     g_static_rw_lock_writer_unlock (&rwlock);
+ *     g_static_rw_lock_writer_unlock (&amp;rwlock);
  *   }
- * ]|
+ *  </programlisting>
+ * </example>
  *
  * This example shows an array which can be accessed by many readers
- * (the my_array_get() function) simultaneously, whereas the writers
- * (the my_array_set() function) will only be allowed once at a time
- * and only if no readers currently access the array. This is because
- * of the potentially dangerous resizing of the array. Using these
- * functions is fully multi-thread safe now.
+ * (the <function>my_array_get()</function> function) simultaneously,
+ * whereas the writers (the <function>my_array_set()</function>
+ * function) will only be allowed once at a time and only if no readers
+ * currently access the array. This is because of the potentially
+ * dangerous resizing of the array. Using these functions is fully
+ * multi-thread safe now.
  *
  * Most of the time, writers should have precedence over readers. That
  * means, for this implementation, that as soon as a writer wants to
@@ -911,24 +911,26 @@ g_static_rec_mutex_free (GStaticRecMutex *mutex)
  * Even though #GStaticRWLock is not opaque, it should only be used
  * with the following functions.
  *
- * All of the g_static_rw_lock_* functions can be used even if
- * g_thread_init() has not been called. Then they do nothing, apart
- * from g_static_rw_lock_*_trylock, which does nothing but returning %TRUE.
+ * All of the <function>g_static_rw_lock_*</function> functions can be
+ * used even if g_thread_init() has not been called. Then they do
+ * nothing, apart from <function>g_static_rw_lock_*_trylock</function>,
+ * which does nothing but returning %TRUE.
  *
- * A read-write lock has a higher overhead than a mutex. For example, both
- * g_static_rw_lock_reader_lock() and g_static_rw_lock_reader_unlock() have
- * to lock and unlock a #GStaticMutex, so it takes at least twice the time
- * to lock and unlock a #GStaticRWLock that it does to lock and unlock a
- * #GStaticMutex. So only data structures that are accessed by multiple
- * readers, and which keep the lock for a considerable time justify a
- * #GStaticRWLock. The above example most probably would fare better with a
- * #GStaticMutex.
+ * <note><para>A read-write lock has a higher overhead than a mutex. For
+ * example, both g_static_rw_lock_reader_lock() and
+ * g_static_rw_lock_reader_unlock() have to lock and unlock a
+ * #GStaticMutex, so it takes at least twice the time to lock and unlock
+ * a #GStaticRWLock that it does to lock and unlock a #GStaticMutex. So
+ * only data structures that are accessed by multiple readers, and which
+ * keep the lock for a considerable time justify a #GStaticRWLock. The
+ * above example most probably would fare better with a
+ * #GStaticMutex.</para></note>
  *
  * Deprecated: 2.32: Use a #GRWLock instead
  **/
 
 /**
- * G_STATIC_RW_LOCK_INIT: (skip):
+ * G_STATIC_RW_LOCK_INIT:
  *
  * A #GStaticRWLock must be initialized with this macro before it can
  * be used. This macro can used be to initialize a variable, but it
@@ -941,12 +943,12 @@ g_static_rec_mutex_free (GStaticRecMutex *mutex)
  */
 
 /**
- * g_static_rw_lock_init: (skip):
+ * g_static_rw_lock_init:
  * @lock: a #GStaticRWLock to be initialized.
  *
  * A #GStaticRWLock must be initialized with this function before it
  * can be used. Alternatively you can initialize it with
- * %G_STATIC_RW_LOCK_INIT.
+ * #G_STATIC_RW_LOCK_INIT.
  *
  * Deprecated: 2.32: Use g_rw_lock_init() instead
  */
@@ -978,7 +980,7 @@ g_static_rw_lock_signal (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_reader_lock: (skip):
+ * g_static_rw_lock_reader_lock:
  * @lock: a #GStaticRWLock to lock for reading.
  *
  * Locks @lock for reading. There may be unlimited concurrent locks for
@@ -1013,8 +1015,9 @@ g_static_rw_lock_reader_lock (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_reader_trylock: (skip):
- * @lock: a #GStaticRWLock to lock for reading
+ * g_static_rw_lock_reader_trylock:
+ * @lock: a #GStaticRWLock to lock for reading.
+ * @Returns: %TRUE, if @lock could be locked for reading.
  *
  * Tries to lock @lock for reading. If @lock is already locked for
  * writing by another thread or if another thread is already waiting to
@@ -1022,9 +1025,7 @@ g_static_rw_lock_reader_lock (GStaticRWLock* lock)
  * @lock for reading and returns %TRUE. This lock has to be unlocked by
  * g_static_rw_lock_reader_unlock().
  *
- * Returns: %TRUE, if @lock could be locked for reading
- *
- * Deprecated: 2.32: Use g_rw_lock_reader_trylock() instead
+ * Deprectated: 2.32: Use g_rw_lock_reader_trylock() instead
  */
 gboolean
 g_static_rw_lock_reader_trylock (GStaticRWLock* lock)
@@ -1047,14 +1048,14 @@ g_static_rw_lock_reader_trylock (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_reader_unlock: (skip):
- * @lock: a #GStaticRWLock to unlock after reading
+ * g_static_rw_lock_reader_unlock:
+ * @lock: a #GStaticRWLock to unlock after reading.
  *
  * Unlocks @lock. If a thread waits to lock @lock for writing and all
  * locks for reading have been unlocked, the waiting thread is woken up
  * and can lock @lock for writing.
  *
- * Deprecated: 2.32: Use g_rw_lock_reader_unlock() instead
+ * Deprectated: 2.32: Use g_rw_lock_reader_unlock() instead
  */
 void
 g_static_rw_lock_reader_unlock  (GStaticRWLock* lock)
@@ -1072,8 +1073,8 @@ g_static_rw_lock_reader_unlock  (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_writer_lock: (skip):
- * @lock: a #GStaticRWLock to lock for writing
+ * g_static_rw_lock_writer_lock:
+ * @lock: a #GStaticRWLock to lock for writing.
  *
  * Locks @lock for writing. If @lock is already locked for writing or
  * reading by other threads, this function will block until @lock is
@@ -1083,7 +1084,7 @@ g_static_rw_lock_reader_unlock  (GStaticRWLock* lock)
  * @lock (neither for reading nor writing). This lock has to be
  * unlocked by g_static_rw_lock_writer_unlock().
  *
- * Deprecated: 2.32: Use g_rw_lock_writer_lock() instead
+ * Deprectated: 2.32: Use g_rw_lock_writer_lock() instead
  */
 void
 g_static_rw_lock_writer_lock (GStaticRWLock* lock)
@@ -1103,17 +1104,16 @@ g_static_rw_lock_writer_lock (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_writer_trylock: (skip):
- * @lock: a #GStaticRWLock to lock for writing
+ * g_static_rw_lock_writer_trylock:
+ * @lock: a #GStaticRWLock to lock for writing.
+ * @Returns: %TRUE, if @lock could be locked for writing.
  *
  * Tries to lock @lock for writing. If @lock is already locked (for
  * either reading or writing) by another thread, it immediately returns
  * %FALSE. Otherwise it locks @lock for writing and returns %TRUE. This
  * lock has to be unlocked by g_static_rw_lock_writer_unlock().
  *
- * Returns: %TRUE, if @lock could be locked for writing
- *
- * Deprecated: 2.32: Use g_rw_lock_writer_trylock() instead
+ * Deprectated: 2.32: Use g_rw_lock_writer_trylock() instead
  */
 gboolean
 g_static_rw_lock_writer_trylock (GStaticRWLock* lock)
@@ -1136,7 +1136,7 @@ g_static_rw_lock_writer_trylock (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_writer_unlock: (skip):
+ * g_static_rw_lock_writer_unlock:
  * @lock: a #GStaticRWLock to unlock after writing.
  *
  * Unlocks @lock. If a thread is waiting to lock @lock for writing and
@@ -1146,7 +1146,7 @@ g_static_rw_lock_writer_trylock (GStaticRWLock* lock)
  * lock @lock for reading, the waiting threads are woken up and can
  * lock @lock for reading.
  *
- * Deprecated: 2.32: Use g_rw_lock_writer_unlock() instead
+ * Deprectated: 2.32: Use g_rw_lock_writer_unlock() instead
  */
 void
 g_static_rw_lock_writer_unlock (GStaticRWLock* lock)
@@ -1163,7 +1163,7 @@ g_static_rw_lock_writer_unlock (GStaticRWLock* lock)
 }
 
 /**
- * g_static_rw_lock_free: (skip):
+ * g_static_rw_lock_free:
  * @lock: a #GStaticRWLock to be freed.
  *
  * Releases all resources allocated to @lock.
@@ -1196,7 +1196,7 @@ g_static_rw_lock_free (GStaticRWLock* lock)
 /* GPrivate {{{1 ------------------------------------------------------ */
 
 /**
- * g_private_new: (skip) (constructor) (not method):
+ * g_private_new:
  * @notify: a #GDestroyNotify
  *
  * Creates a new #GPrivate.
@@ -1204,7 +1204,7 @@ g_static_rw_lock_free (GStaticRWLock* lock)
  * Deprecated:2.32: dynamic allocation of #GPrivate is a bad idea.  Use
  *                  static storage and G_PRIVATE_INIT() instead.
  *
- * Returns: (transfer full): a newly allocated #GPrivate (which can never be destroyed)
+ * Returns: a newly allocated #GPrivate (which can never be destroyed)
  */
 GPrivate *
 g_private_new (GDestroyNotify notify)
@@ -1247,37 +1247,41 @@ g_static_private_cleanup (gpointer data)
 GPrivate static_private_private = G_PRIVATE_INIT (g_static_private_cleanup);
 
 /**
- * GStaticPrivate: (skip):
+ * GStaticPrivate:
  *
  * A #GStaticPrivate works almost like a #GPrivate, but it has one
  * significant advantage. It doesn't need to be created at run-time
  * like a #GPrivate, but can be defined at compile-time. This is
- * similar to the difference between #GMutex and #GStaticMutex.
+ * similar to the difference between #GMutex and #GStaticMutex. Now
+ * look at our <function>give_me_next_number()</function> example with
+ * #GStaticPrivate:
  *
- * Now look at our give_me_next_number() example with #GStaticPrivate:
- * |[
+ * <example>
+ *  <title>Using GStaticPrivate for per-thread data</title>
+ *  <programlisting>
  *   int
- *   give_me_next_number ()
+ *   give_me_next_number (<!-- -->)
  *   {
  *     static GStaticPrivate current_number_key = G_STATIC_PRIVATE_INIT;
- *     int *current_number = g_static_private_get (&current_number_key);
+ *     int *current_number = g_static_private_get (&amp;current_number_key);
  *
  *     if (!current_number)
  *       {
- *         current_number = g_new (int, 1);
+ *         current_number = g_new (int,1);
  *         *current_number = 0;
- *         g_static_private_set (&current_number_key, current_number, g_free);
+ *         g_static_private_set (&amp;current_number_key, current_number, g_free);
  *       }
  *
  *     *current_number = calc_next_number (*current_number);
  *
  *     return *current_number;
  *   }
- * ]|
+ *  </programlisting>
+ * </example>
  */
 
 /**
- * G_STATIC_PRIVATE_INIT: (skip):
+ * G_STATIC_PRIVATE_INIT:
  *
  * Every #GStaticPrivate must be initialized with this macro, before it
  * can be used.
@@ -1288,11 +1292,11 @@ GPrivate static_private_private = G_PRIVATE_INIT (g_static_private_cleanup);
  */
 
 /**
- * g_static_private_init: (skip):
+ * g_static_private_init:
  * @private_key: a #GStaticPrivate to be initialized
  *
  * Initializes @private_key. Alternatively you can initialize it with
- * %G_STATIC_PRIVATE_INIT.
+ * #G_STATIC_PRIVATE_INIT.
  */
 void
 g_static_private_init (GStaticPrivate *private_key)
@@ -1301,7 +1305,7 @@ g_static_private_init (GStaticPrivate *private_key)
 }
 
 /**
- * g_static_private_get: (skip):
+ * g_static_private_get:
  * @private_key: a #GStaticPrivate
  *
  * Works like g_private_get() only for a #GStaticPrivate.
@@ -1344,7 +1348,7 @@ g_static_private_get (GStaticPrivate *private_key)
 }
 
 /**
- * g_static_private_set: (skip):
+ * g_static_private_set:
  * @private_key: a #GStaticPrivate
  * @data: the new pointer
  * @notify: a function to be called with the pointer whenever the
@@ -1360,7 +1364,8 @@ g_static_private_get (GStaticPrivate *private_key)
  * will be inherited only by the main thread, i.e. the one that called
  * g_thread_init().
  *
- * @notify is used quite differently from @destructor in g_private_new().
+ * <note><para>@notify is used quite differently from @destructor in
+ * g_private_new().</para></note>
  */
 void
 g_static_private_set (GStaticPrivate *private_key,
@@ -1410,7 +1415,7 @@ g_static_private_set (GStaticPrivate *private_key,
 }
 
 /**
- * g_static_private_free: (skip):
+ * g_static_private_free:
  * @private_key: a #GStaticPrivate to be freed
  *
  * Releases all resources allocated to @private_key.
@@ -1443,13 +1448,13 @@ g_static_private_free (GStaticPrivate *private_key)
 /* GMutex {{{1 ------------------------------------------------------ */
 
 /**
- * g_mutex_new: (skip) (constructor) (not method):
+ * g_mutex_new:
  *
  * Allocates and initializes a new #GMutex.
  *
- * Returns: (transfer full): a newly allocated #GMutex. Use g_mutex_free() to free
+ * Returns: a newly allocated #GMutex. Use g_mutex_free() to free
  *
- * Deprecated: 2.32: GMutex can now be statically allocated, or embedded
+ * Deprecated:3.32:GMutex can now be statically allocated, or embedded
  * in structures and initialised with g_mutex_init().
  */
 GMutex *
@@ -1464,7 +1469,7 @@ g_mutex_new (void)
 }
 
 /**
- * g_mutex_free: (skip):
+ * g_mutex_free:
  * @mutex: a #GMutex
  *
  * Destroys a @mutex that has been created with g_mutex_new().
@@ -1472,7 +1477,7 @@ g_mutex_new (void)
  * Calling g_mutex_free() on a locked mutex may result
  * in undefined behaviour.
  *
- * Deprecated: 2.32: GMutex can now be statically allocated, or embedded
+ * Deprecated:3.32:GMutex can now be statically allocated, or embedded
  * in structures and initialised with g_mutex_init().
  */
 void
@@ -1485,13 +1490,13 @@ g_mutex_free (GMutex *mutex)
 /* GCond {{{1 ------------------------------------------------------ */
 
 /**
- * g_cond_new: (skip) (constructor) (not method):
+ * g_cond_new:
  *
  * Allocates and initializes a new #GCond.
  *
- * Returns: (transfer full): a newly allocated #GCond. Free with g_cond_free()
+ * Returns: a newly allocated #GCond. Free with g_cond_free()
  *
- * Deprecated: 2.32: GCond can now be statically allocated, or embedded
+ * Deprecated:3.32:GCond can now be statically allocated, or embedded
  * in structures and initialised with g_cond_init().
  */
 GCond *
@@ -1506,7 +1511,7 @@ g_cond_new (void)
 }
 
 /**
- * g_cond_free: (skip):
+ * g_cond_free:
  * @cond: a #GCond
  *
  * Destroys a #GCond that has been created with g_cond_new().
@@ -1514,7 +1519,7 @@ g_cond_new (void)
  * Calling g_cond_free() for a #GCond on which threads are
  * blocking leads to undefined behaviour.
  *
- * Deprecated: 2.32: GCond can now be statically allocated, or embedded
+ * Deprecated:3.32:GCond can now be statically allocated, or embedded
  * in structures and initialised with g_cond_init().
  */
 void
@@ -1525,7 +1530,7 @@ g_cond_free (GCond *cond)
 }
 
 /**
- * g_cond_timed_wait: (skip):
+ * g_cond_timed_wait:
  * @cond: a #GCond
  * @mutex: a #GMutex that is currently locked
  * @abs_time: a #GTimeVal, determining the final time
@@ -1539,11 +1544,10 @@ g_cond_free (GCond *cond)
  * This function can be used even if g_thread_init() has not yet been
  * called, and, in that case, will immediately return %TRUE.
  *
- * To easily calculate @abs_time a combination of g_get_real_time()
+ * To easily calculate @abs_time a combination of g_get_current_time()
  * and g_time_val_add() can be used.
  *
  * Returns: %TRUE if @cond was signalled, or %FALSE on timeout
- *
  * Deprecated:2.32: Use g_cond_wait_until() instead.
  */
 gboolean
@@ -1563,10 +1567,16 @@ g_cond_timed_wait (GCond    *cond,
   end_time *= 1000000;
   end_time += abs_time->tv_usec;
 
+#ifdef CLOCK_MONOTONIC
   /* would be nice if we had clock_rtoffset, but that didn't seem to
    * make it into the kernel yet...
    */
   end_time += g_get_monotonic_time () - g_get_real_time ();
+#else
+  /* if CLOCK_MONOTONIC is not defined then g_get_montonic_time() and
+   * g_get_real_time() are returning the same clock, so don't bother...
+   */
+#endif
 
   return g_cond_wait_until (cond, mutex, end_time);
 }
