@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -22,19 +24,17 @@
  * GLib at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __G_DEPRECATED_THREAD_H__
-#define __G_DEPRECATED_THREAD_H__
-
 #if !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
 #error "Only <glib.h> can be included directly."
 #endif
 
+#ifndef __G_DEPRECATED_THREAD_H__
+#define __G_DEPRECATED_THREAD_H__
+
 #include <glib/gthread.h>
 
 G_BEGIN_DECLS
 
-#ifndef G_DISABLE_DEPRECATED
-
 typedef enum
 {
   G_THREAD_PRIORITY_LOW,
@@ -43,8 +43,6 @@ typedef enum
   G_THREAD_PRIORITY_URGENT
 } GThreadPriority;
 
-#endif
-
 struct  _GThread
 {
   /*< private >*/
@@ -54,8 +52,6 @@ struct  _GThread
   GThreadPriority priority;
 };
 
-#ifndef G_DISABLE_DEPRECATED
-
 typedef struct _GThreadFunctions GThreadFunctions;
 struct _GThreadFunctions
 {
@@ -100,13 +96,13 @@ GLIB_VAR gboolean               g_thread_use_default_impl;
 
 GLIB_VAR guint64   (*g_thread_gettime) (void);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_thread_new)
+GLIB_DEPRECATED_FOR(g_thread_new)
 GThread *g_thread_create       (GThreadFunc       func,
                                 gpointer          data,
                                 gboolean          joinable,
                                 GError          **error);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_thread_new)
+GLIB_DEPRECATED_FOR(g_thread_new)
 GThread *g_thread_create_full  (GThreadFunc       func,
                                 gpointer          data,
                                 gulong            stack_size,
@@ -115,16 +111,15 @@ GThread *g_thread_create_full  (GThreadFunc       func,
                                 GThreadPriority   priority,
                                 GError          **error);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_thread_set_priority (GThread          *thread,
                                 GThreadPriority   priority);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_thread_foreach      (GFunc             thread_func,
                                 gpointer          user_data);
 
 #ifndef G_OS_WIN32
-#include <sys/types.h>
 #include <pthread.h>
 #endif
 
@@ -146,11 +141,10 @@ typedef struct
 #define g_static_mutex_unlock(mutex) \
     g_mutex_unlock (g_static_mutex_get_mutex (mutex))
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_mutex_init)
+GLIB_DEPRECATED_FOR(g_mutex_init)
 void    g_static_mutex_init           (GStaticMutex *mutex);
-GLIB_DEPRECATED_IN_2_32_FOR(g_mutex_clear)
+GLIB_DEPRECATED_FOR(g_mutex_free)
 void    g_static_mutex_free           (GStaticMutex *mutex);
-GLIB_DEPRECATED_IN_2_32_FOR(GMutex)
 GMutex *g_static_mutex_get_mutex_impl (GStaticMutex *mutex);
 
 typedef struct _GStaticRecMutex GStaticRecMutex;
@@ -168,30 +162,30 @@ struct _GStaticRecMutex
     pthread_t owner;
 #endif
     gdouble dummy;
-  } unused;
+  };
 };
 
 #define G_STATIC_REC_MUTEX_INIT { G_STATIC_MUTEX_INIT }
-GLIB_DEPRECATED_IN_2_32_FOR(g_rec_mutex_init)
+GLIB_DEPRECATED_FOR(g_rec_mutex_init)
 void     g_static_rec_mutex_init        (GStaticRecMutex *mutex);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rec_mutex_lock)
+GLIB_DEPRECATED_FOR(g_rec_mutex_lock)
 void     g_static_rec_mutex_lock        (GStaticRecMutex *mutex);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rec_mutex_try_lock)
+GLIB_DEPRECATED_FOR(g_rec_mutex_try_lock)
 gboolean g_static_rec_mutex_trylock     (GStaticRecMutex *mutex);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rec_mutex_unlock)
+GLIB_DEPRECATED_FOR(g_rec_mutex_unlock)
 void     g_static_rec_mutex_unlock      (GStaticRecMutex *mutex);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_static_rec_mutex_lock_full   (GStaticRecMutex *mutex,
                                          guint            depth);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 guint    g_static_rec_mutex_unlock_full (GStaticRecMutex *mutex);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rec_mutex_free)
+GLIB_DEPRECATED_FOR(g_rec_mutex_free)
 void     g_static_rec_mutex_free        (GStaticRecMutex *mutex);
 
 typedef struct _GStaticRWLock GStaticRWLock;
@@ -209,31 +203,31 @@ struct _GStaticRWLock
 
 #define G_STATIC_RW_LOCK_INIT { G_STATIC_MUTEX_INIT, NULL, NULL, 0, FALSE, 0, 0 }
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_init)
+GLIB_DEPRECATED_FOR(g_rw_lock_init)
 void      g_static_rw_lock_init           (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_reader_lock)
+GLIB_DEPRECATED_FOR(g_rw_lock_reader_lock)
 void      g_static_rw_lock_reader_lock    (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_reader_trylock)
+GLIB_DEPRECATED_FOR(g_rw_lock_reader_trylock)
 gboolean  g_static_rw_lock_reader_trylock (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_reader_unlock)
+GLIB_DEPRECATED_FOR(g_rw_lock_reader_unlock)
 void      g_static_rw_lock_reader_unlock  (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_writer_lock)
+GLIB_DEPRECATED_FOR(g_rw_lock_writer_lock)
 void      g_static_rw_lock_writer_lock    (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_writer_trylock)
+GLIB_DEPRECATED_FOR(g_rw_lock_writer_trylock)
 gboolean  g_static_rw_lock_writer_trylock (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_writer_unlock)
+GLIB_DEPRECATED_FOR(g_rw_lock_writer_unlock)
 void      g_static_rw_lock_writer_unlock  (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_rw_lock_free)
+GLIB_DEPRECATED_FOR(g_rw_lock_free)
 void      g_static_rw_lock_free           (GStaticRWLock *lock);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 GPrivate *      g_private_new             (GDestroyNotify notify);
 
 typedef struct _GStaticPrivate  GStaticPrivate;
@@ -244,50 +238,48 @@ struct _GStaticPrivate
 };
 
 #define G_STATIC_PRIVATE_INIT { 0 }
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_static_private_init           (GStaticPrivate *private_key);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_private_get)
+GLIB_DEPRECATED_FOR(g_private_get)
 gpointer g_static_private_get            (GStaticPrivate *private_key);
 
-GLIB_DEPRECATED_IN_2_32_FOR(g_private_set)
+GLIB_DEPRECATED_FOR(g_private_set)
 void     g_static_private_set            (GStaticPrivate *private_key,
                                           gpointer        data,
                                           GDestroyNotify  notify);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_static_private_free           (GStaticPrivate *private_key);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 gboolean g_once_init_enter_impl          (volatile gsize *location);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void     g_thread_init                   (gpointer vtable);
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void    g_thread_init_with_errorcheck_mutexes (gpointer vtable);
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 gboolean g_thread_get_initialized        (void);
 
 GLIB_VAR gboolean g_threads_got_initialized;
 
 #define g_thread_supported()     (1)
 
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 GMutex *        g_mutex_new             (void);
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void            g_mutex_free            (GMutex *mutex);
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 GCond *         g_cond_new              (void);
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 void            g_cond_free             (GCond  *cond);
-GLIB_DEPRECATED_IN_2_32
+GLIB_DEPRECATED
 gboolean        g_cond_timed_wait       (GCond          *cond,
                                          GMutex         *mutex,
                                          GTimeVal       *timeval);
 
-#endif
-
 G_END_DECLS
 
 #endif /* __G_DEPRECATED_THREAD_H__ */
