@@ -34,11 +34,12 @@
 #include "ghash.h"
 #include "gthread.h"
 #include "gbufferprivate.h"
+#include "gslice.h"
 
 /**
  * SECTION:timezone
  * @title: GTimeZone
- * @short_description: A structure representing a time zone
+ * @short_description: a structure representing a time zone
  * @see_also: #GDateTime
  *
  * #GTimeZone is a structure that represents a time zone, at no
@@ -366,10 +367,6 @@ g_time_zone_new (const gchar *identifier)
         {
           gchar *filename;
 
-	  /* identifier can be a relative or absolute path name;
-	     if relative, it is interpreted starting from /usr/share/timezone
-	     while the POSIX standard says it should start with :,
-	     glibc allows both syntaxes, so we should too */
           if (identifier != NULL)
             {
               const gchar *tzdir;
@@ -378,13 +375,7 @@ g_time_zone_new (const gchar *identifier)
               if (tzdir == NULL)
                 tzdir = "/usr/share/zoneinfo";
 
-	      if (*identifier == ':')
-		identifier ++;
-
-	      if (g_path_is_absolute (identifier))
-		filename = g_strdup (identifier);
-	      else
-		filename = g_build_filename (tzdir, identifier, NULL);
+              filename = g_build_filename (tzdir, identifier, NULL);
             }
           else
             filename = g_strdup ("/etc/localtime");
