@@ -21,6 +21,10 @@
 
 #include <string.h>
 #include <errno.h>
+
+/* We are testing some deprecated APIs here */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <glib.h>
 #include <glib/gstdio.h>
 
@@ -478,6 +482,9 @@ test_mkdir_with_parents (void)
     g_print ("checking g_mkdir_with_parents() in cwd: %s", cwd);
   test_mkdir_with_parents_1 (cwd);
   g_free (cwd);
+
+  g_assert (g_mkdir_with_parents (NULL, 0) == -1);
+  g_assert (errno == EINVAL);
 }
 
 static void
