@@ -444,7 +444,6 @@ test_listing (void)
   gsize len;
   gchar *start;
   GError *error = NULL;
-  gboolean has_key;
 
   const gchar *data =
     "[group1]\n"
@@ -497,21 +496,6 @@ test_listing (void)
   g_key_file_has_key (keyfile, "no-such-group", "key", &error);
   check_error (&error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND);
 
-  g_assert (g_key_file_has_key_full (keyfile, "group1", "key1", &has_key, &error));
-  check_no_error (&error);
-  g_assert (has_key);
-
-  g_assert (g_key_file_has_key_full (keyfile, "group2", "key3", &has_key, &error));
-  check_no_error (&error);
-  g_assert (has_key);
-
-  g_assert (g_key_file_has_key_full (keyfile, "group2", "no-such-key", &has_key, &error));
-  g_assert (!has_key);
-  check_no_error (&error);
-
-  g_assert (!g_key_file_has_key_full (keyfile, "no-such-group", "key", &has_key, &error));
-  check_error (&error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND);
-
   g_key_file_free (keyfile);
 }
 
@@ -1414,6 +1398,25 @@ test_page_boundary (void)
 
 }
 
+static void
+test_ref (void)
+{
+  GKeyFile *file;
+  static const char data[] =
+"[group]\n"
+"a=1\n";
+  gboolean ok;
+
+  file = g_key_file_new ();
+
+  ok = g_key_file_load_from_data (file, data, strlen (data), 0, NULL);
+  g_assert (ok);
+  g_assert (g_key_file_has_key (file, "group", "a", NULL));
+  g_key_file_ref (file);
+  g_key_file_free (file);
+  g_key_file_unref (file);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1445,6 +1448,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/keyfile/load", test_load);
   g_test_add_func ("/keyfile/non-utf8", test_non_utf8);
   g_test_add_func ("/keyfile/page-boundary", test_page_boundary);
+  g_test_add_func ("/keyfile/ref", test_ref);
 
   return g_test_run ();
 }
